// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Stage.h"
#include "MyCtrl.h"
#include "maze.h"
#include "indevdlg.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ACTIVATEAPP()
	ON_WM_PALETTECHANGED()
	ON_WM_MOVE()
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_WM_SETFOCUS()
	ON_COMMAND(ID_EDIT_INPUTDEV, OnEditInputdev)
	ON_COMMAND(ID_EDIT_CTRLDEV, OnEditCtrldev)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pScene = NULL;
	m_pController = NULL;
	m_iObjCtrlDev = 0;
	m_pSoldier1 = NULL;
	m_pSoldier2 = NULL;
	m_pBlast1 = NULL;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Create the 3D window
	if (!m_wnd3d.Create(this, IDC_3DWND)) {
		return -1;
	}

	NewScene();
	ASSERT(m_pScene);

	// Create the controller object
	m_pController = new CMyCtrl(this);
	m_pController->Create(&m_wnd3d,
						  OnGetCtrlFrame,
					      this);

	// restore the controller settings
	m_pController->SelectDevice(2);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// Restore any saved settings
	RestoreSettings();

	// Restore the window size and position
	cs.x = m_rcWnd.left;
	cs.y = m_rcWnd.top;
	cs.cx = m_rcWnd.Width();
	cs.cy = m_rcWnd.Height();

	return CFrameWnd::PreCreateWindow(cs);
}

BOOL CMainFrame::NewScene()
{
	// Delete any scene we might have
	if (m_pScene) {

		m_wnd3d.SetScene(NULL);
		delete m_pScene;
		m_pScene = NULL;
	}

	// Create an initial scene
	m_pScene = new C3dScene;
	if (!m_pScene->Create()) return FALSE;

	// Set up the lighting
    C3dDirLight dl;
    dl.Create(0.8, 0.8, 0.8);
    m_pScene->AddChild(&dl);
    dl.SetPosition(-2, 2, -5);
    dl.SetDirection(1, -1, 1);
    m_pScene->SetAmbientLight(0.6, 0.6, 0.6);

	m_wnd3d.SetScene(m_pScene);

	// Set the initial camera position and direction
	m_pScene->SetCameraPosition(C3dVector(4, 0.3, -2));
	m_pScene->SetCameraDirection(C3dVector(0, 0, 1));
	m_pScene->SetCameraField(0.2);
	
	// Set up the scenery
	m_pScene->SetBackground(0, 0, 1);

	// Build the maze
	CFloor* pFloor = new CFloor(-6, -6, 6, 6);
	m_pScene->AddChild(pFloor);
	m_pScene->m_ShapeList.Append(pFloor);

	WALLINFO wi [] = {
		{-6,  5,  6,  6, 1},
		{ 5, -5,  6,  5, 0.9},
		{-6, -6,  6, -5, 1.1},
		{-6, -5, -5,  5, 0.8},
		{-3,  2,  2,  3, 0.5},
		{ 2, -3,  3,  3, 0.9},
		{-4, -1,  0,  0, 1},
		{-1, -5,  0, -2, 1.3}

	};
	int nWalls = sizeof(wi) / sizeof(WALLINFO);

	WALLINFO* pwi = wi;
	for (int i = 0; i < nWalls; i++) {
		CWall* pWall = new CWall(pwi);
		m_pScene->AddChild(pWall);
		m_pScene->m_ShapeList.Append(pWall);
		pwi++;
	}

	// Add the soldiers
	m_pSoldier1 = new CSoldier(m_pScene, -3, 4);
	m_pSoldier2 = new CSoldier(m_pScene, -3, -3);

	// Add a blast area
	m_pBlast1 = new CBlast(m_pScene, 4, 4);

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::RecalcLayout(BOOL bNotify) 
{
    // Rearrange the control bars and fit the 3D window in the middle
	// Let the frame rearrange the control bars
	CFrameWnd::RecalcLayout(bNotify);

	// Find the space that's left over
	CRect rc;
	RepositionBars(0,
				   0xFFFF,
				   IDC_3DWND,
				   CWnd::reposQuery,
				   &rc);
	if (IsWindow(m_wnd3d.GetSafeHwnd())) {
		m_wnd3d.MoveWindow(&rc, FALSE);
	}
}


void CMainFrame::OnDestroy() 
{
	CFrameWnd::OnDestroy();

	// Save the current settings
	SaveSettings();
	
	// destroy the controller
	if (m_pController) {
		delete m_pController;
		m_pController = NULL;
	}

	// Delete any scene we might have
	m_wnd3d.SetScene(NULL);
	if (m_pScene) {
		delete m_pScene;
		m_pScene = NULL;
	}
}

void CMainFrame::OnActivateApp(BOOL bActive, HTASK hTask) 
{
	CFrameWnd::OnActivateApp(bActive, hTask);
	
	// Tell the 3D window about the new state
	m_wnd3d.SendMessage(WM_ACTIVATEAPP,
						(WPARAM)bActive,
						(LPARAM)hTask);
}

void CMainFrame::OnPaletteChanged(CWnd* pFocusWnd) 
{
	// Let the 3D window know
	m_wnd3d.SendMessage(WM_PALETTECHANGED,
					   pFocusWnd ? (WPARAM)pFocusWnd->GetSafeHwnd() : 0);
}

void CMainFrame::OnMove(int x, int y) 
{
	CFrameWnd::OnMove(x, y);
	
	// Let the 3D window know the frame has moved
	m_wnd3d.SendMessage(WM_MOVE, 0, MAKELPARAM(0, 0));
}

void CMainFrame::OnFileNew() 
{
	NewScene();
}

BOOL CMainFrame::OpenFile(const char* pszPath)
{
	return FALSE;
}

void CMainFrame::OnSetFocus(CWnd* pOldWnd) 
{
	CFrameWnd::OnSetFocus(pOldWnd);
	
	// Give the focus to the 3D window so it can process
	// keystrokes for the key input device
	m_wnd3d.SetFocus();
}

//////////////////////////////////////////////////////////////
// Control device selection

// callback from the dialog to get a frame to control
// static
C3dFrame* CMainFrame::OnGetCtrlFrame(void* pArg)
{
	CMainFrame* pThis = (CMainFrame*) pArg;
	ASSERT(pThis);
	ASSERT(pThis->IsKindOf(RUNTIME_CLASS(CMainFrame)));
	return pThis->m_wnd3d.GetStage()->GetCamera();
}

void CMainFrame::OnEditInputdev() 
{
	ASSERT(m_pController);
	C3dInputDevice* pDev = m_pController->GetInputDevice();
	ASSERT(pDev);
	pDev->ShowConfigDlg();
}

void CMainFrame::OnEditCtrldev() 
{
	CInDevDlg dlg;
	dlg.m_iDevice = m_iObjCtrlDev;
	if (dlg.DoModal() != IDOK) return;
	m_iObjCtrlDev = dlg.m_iDevice;
	ASSERT(m_pController);
	m_pController->SelectDevice(m_iObjCtrlDev);
}

//////////////////////////////////////////////////////////////////
// Save and Restore Settings

void CMainFrame::SaveSettings()
{
	CWinApp* pa = AfxGetApp();
	pa->WriteProfileInt("Settings","ControlDevice", m_iObjCtrlDev);
	CRect rc;
	GetWindowRect(&rc);
	pa->WriteProfileInt("Settings\\Window","Top", rc.top);
	pa->WriteProfileInt("Settings\\Window","Left", rc.left);
	pa->WriteProfileInt("Settings\\Window","Width", rc.Width());
	pa->WriteProfileInt("Settings\\Window","Height", rc.Height());
}

void CMainFrame::RestoreSettings()
{
	CWinApp* pa = AfxGetApp();
	m_iObjCtrlDev = pa->GetProfileInt("Settings","ControlDevice", 0);
	m_rcWnd.top = pa->GetProfileInt("Settings\\Window","Top", 50);
	m_rcWnd.left = pa->GetProfileInt("Settings\\Window","Left", 50);
	m_rcWnd.right = m_rcWnd.left + pa->GetProfileInt("Settings\\Window","Width", 300); 
	m_rcWnd.bottom = m_rcWnd.top + pa->GetProfileInt("Settings\\Window","Height", 350); 
}

BOOL CMainFrame::Update()
{
	// Move the soldiers
	if (m_pSoldier1) m_pSoldier1->Update();
	if (m_pSoldier2) m_pSoldier2->Update();

	// Cycle the explostions
	if (m_pBlast1) m_pBlast1->Update();

	// see if the object controller wants to do anything
	if (m_pController) {
		m_pController->Update();
	}

	// tell the 3D window to update
	return m_wnd3d.Update(1);
}