// maze.cpp
//
// Implementation for CFloor, CWall, CSoldier, CBlast
//

#include "stdafx.h"
#include "Stage.h"
#include "maze.h"

///////////////////////////////////////////////////////////////////////////
// CFloor

CFloor::CFloor(double x1, double z1, double x2, double z2)
{
	// create a shape with only one face and apply
	// a texture map to the face

    D3DVECTOR vlist[] = {
        {x1, 0.0, z1},
        {x1, 0.0, z2},
        {x2, 0.0, z2},
        {x2, 0.0, z1}
    };
    int iVectors = sizeof(vlist) / sizeof(D3DVECTOR);
    int iFaces[] = {4, 0, 1, 2, 3,  
                    0};

    Create(vlist, iVectors, iFaces);

	// Load the texture
	C3dTexture* pTex = new C3dTexture;
	if (!pTex->Load(IDB_FLOOR)) {
		return;
	}
	m_ImgList.Append(pTex);

	// Attach the texture
	SetTexture(pTex);

	// Create a wrap
	C3dWrap wrap;
    wrap.Create(D3DRMWRAP_FLAT,
				NULL,
				0, 0, 0, // origin
				0, 1, 0, // direction
				0, 0, 1, // up
				0, 0,	 // texture origin
				1, 1);	 // texture scale 

	wrap.Apply(this);
	SetName("Floor");
}



///////////////////////////////////////////////////////////////////////////
// CWall

// static
C3dTexture* CWall::s_pTex = NULL;
// static
int CWall::s_iWallCount = 0;

CWall::CWall(WALLINFO* pInfo)
{
	// Create 4 walls with separate faces

    double x1 = pInfo->x1;
	double x2 = pInfo->x2;
	double z1 = pInfo->z1;
	double z2 = pInfo->z2;
	double h = pInfo->h;

	D3DVECTOR vlist[] = {
        {x1, 0, z1},
        {x1, 0, z2},
        {x1, h, z2},
        {x1, h, z1}, // left

        {x2, 0, z1},
        {x2, h, z1},
        {x2, h, z2},
        {x2, 0, z2}, // right

        {x1, 0, z1},
        {x1, h, z1},
        {x2, h, z1},
        {x2, 0, z1}, // front

        {x2, 0, z2},
        {x2, h, z2},
        {x1, h, z2},
        {x1, 0, z2}  // back

    };
    int iVectors = sizeof(vlist) / sizeof(D3DVECTOR);
    int iFaces[] = {4, 0, 1, 2, 3,
                    4, 4, 5, 6, 7,
					4, 8, 9, 10, 11,
					4, 12, 13, 14, 15,
                    0};

    Create(vlist, iVectors, iFaces);

	// load the texture map
	if (!s_pTex) {
		s_pTex = new C3dTexture;
		s_pTex->Load(IDB_WALL);
	}

	for (int i = 0; i < 4; i++) {

		// Attach the texture to the face
		SetFaceTexture(i, s_pTex);

		// Get the face normal
		C3dVector vn = GetFaceNormal(i);

		// reverse it so it represents the wrap direction
		vn = -vn;

		// Compute an arbitrary up vector
		C3dVector vu = vn.GenerateUp();

		// Create a wrap oriented to this face
		C3dWrap wrap;
		wrap.Create(D3DRMWRAP_FLAT,
					NULL,
					0, 0, 0,		  // origin
					vn.x, vn.y, vn.z, // direction
					vu.x, vu.y, vu.z, // up
					0, 0,			  // texture origin
					1, 1);			  // texture scale

		// apply the wrap to the face
		wrap.Apply(this, i);
	}

	SetName("Wall");
	s_iWallCount++;
}

CWall::~CWall()
{
	s_iWallCount--;
	if (s_iWallCount == 0) {
		delete s_pTex;
		s_pTex = NULL;
	}
}

///////////////////////////////////////////////////////////////////////////
// CSoldier

CSoldier::CSoldier(C3dScene* pScene, double x, double z)
{
	Create(pScene, x, 0, z, 0.5, IDB_SOLDIER, 4);
}

void CSoldier::Update()
{
	SetPhase((GetPhase() + 1) % GetNumPhases());
}

////////////////////////////////////////////////////////////////////////////
// CBlast

CBlast::CBlast(C3dScene* pScene, double x, double z)
{
	Create(pScene, x, -0.35, z, 1.0, IDB_BLAST, 5);
}

void CBlast::Explode()
{
	SetPhase(1);
}

void CBlast::Update()
{
	int iPhase = GetPhase();
	if (iPhase == 0) return;
	int iPhases = GetNumPhases();
	if (iPhase < iPhases - 1) {
		SetPhase(iPhase + 1);
	}
}



